! module for handling input files and define hamiltonian and integration parameters
module parameters

  implicit none

  !-------------------------------------------------------------------------
  ! Parameters required for computing the Hamiltonian function
  ! 
  type hamparam

     double precision:: m0
     double precision:: mBod(2)
     double precision:: Gm0mi(2)
     double precision:: delta
     double precision:: ddelta

  end type hamparam
!-------------------------------------------------------------------------
  ! Parameters of the numerical integration
  ! 
  type intparam

     double precision:: t0    ! initial time
     double precision:: tSpan ! duration
     double precision:: Dt    ! time step
     double precision:: tfac  ! time factor normalized hamiltonian
     integer:: Nsave          ! number of timesteps between each save

  end type intparam

!-------------------------------------------------------------------------
contains
!***************************************************************************************

  ! Read the hamiltonian parameters from an input file and rewrite them in an output file.
  ! All the parameters should have the same system of units (e.g. au, years, Sun-mass).
  ! 
  subroutine readHamParam(ficIn,ficOut,Hp)
     character(len=*),intent(in):: ficIn, ficOut
     type(hamparam),intent(out):: Hp

     double precision:: Gconst        ! gravitational constant
     double precision:: m0            ! mass of central body
     double precision:: mBod(2)       ! masses of the bodies
     double precision:: dota          ! change rate of semi-major axis due to dissipation 

     integer:: Nreg, i

     namelist /physicalparams/ Gconst, m0, mBod, dota

     !-----------------------------------------------------------------------
     ! Reading the parameters
     open(newunit=Nreg,file=trim(ficIn),status="OLD",action="READ")
     read(Nreg,physicalparams)
     close(Nreg)

     ! Writing the parameters
     open(newunit=Nreg,file=trim(ficOut),status="REPLACE",action="WRITE")

     write(Nreg,'(A)')              "&physicalparams"
     write(Nreg,'(A,ES22.15,A)')    " Gconst = " , Gconst,","
     write(Nreg,'(A,ES22.15,A)')    "     m0 = " , m0,","
     write(Nreg,'(A,*(ES22.15,A))') "   mBod = " , (mBod(i),",",i=1,2)
     write(Nreg,'(A,*(ES22.15,A))') "   dota = " , dota
     write(Nreg,'(A)') "/"

     close(Nreg)
    
     !-----------------------------------------------------------------------
     Hp%m0 = m0
     Hp%mBod(1) = mBod(1)
     Hp%mBod(2) = mBod(2)
     Hp%Gm0mi(1) = Gconst*(m0+mBod(1))
     Hp%Gm0mi(2) = Gconst*(m0+mBod(2))

     ! I need initial conditions
     Hp%delta = 0.d0
     Hp%ddelta = dota     ! store dota

  end subroutine readHamParam

!***************************************************************************************

  ! Read the parameters of the numerical integration and the initial conditions in an
  ! input file, and rewrite them in an output file.
  ! 
  subroutine initInteg(ficIn,ficOut, Hp,Ip, Z0)
     character(len=*),intent(in):: ficIn, ficOut
     type(hamparam),intent(inout):: Hp
     type(intparam),intent(out):: Ip
     double precision,intent(out):: Z0(2)


     double precision:: t0    ! initial time
     double precision:: tSpan ! duration
     double precision:: Dt    ! time step
     double precision:: tfac    ! time step
     integer:: Nsave          ! number of timesteps between each save

     double precision:: X(6,2)
     double precision:: Gamma, res
     
     double precision:: alpha, beta, epsi, eta, n1, n2, fd
     
     integer:: Nreg, i, j

     namelist /integrationparam/ t0, tSpan, Dt, tfac, Nsave, X

     !-----------------------------------------------------------------------
     ! Reading the parameters
     open(newunit=Nreg,file=trim(ficIn),status="OLD",action="READ")
     read(Nreg,integrationparam)
     close(Nreg)

     ! Writing the parameters
     open(newunit=Nreg,file=trim(ficOut),status="REPLACE",action="WRITE")

     write(Nreg,'(A)')              "&integrationparam"
     write(Nreg,'(A,ES22.15,A)')    "     t0 = " , t0,","
     write(Nreg,'(A,ES22.15,A)')    "  tSpan = " , tSpan,","
     write(Nreg,'(A,ES22.15,A)')    "     Dt = " , Dt,","
     write(Nreg,'(A,I0,A)')         "  Nsave =  ", Nsave,","
     do j=1,2
        write(Nreg,'(A,*(ES22.15,A))') "      X = " , (X(i,j),",",i=1,6)
     endDo
     write(Nreg,'(A)') "/"

     close(Nreg)

     !----------------------------------------------------------------------
     ! update Hp parameters using initial conditions of semi-major axes
     fd = -1.19d0
     n1 = sqrt(Hp%Gm0mi(1)/X(1,1)**3)
     n2 = sqrt(Hp%Gm0mi(2)/X(1,2)**3)

     alpha = n1 - 2.d0*n2
     beta = 1.5d0*( 1.d0/(Hp%mBod(1)*X(1,1)**2) + 4.d0/(Hp%mBod(2)*X(1,2)**2) )
     epsi = n1**(1.5d0)*fd*X(1,1)**2/X(1,2)*Hp%mBod(2)/Hp%m0*Hp%mBod(1)**(0.5d0)
     eta =  (-2.d0*beta/epsi)**(-2.d0/3.d0)
     
     Hp%delta = alpha/beta/eta
     Hp%ddelta = -1.5d0*n1/X(1,1)/beta/eta*Hp%ddelta      ! only contribution of alpha for simplicity
     Hp%ddelta = Hp%ddelta/eta/beta ! rescaled time

     !-----------------------------------------------------------------------
     ! initial conditions for the canonical coordinates
     Gamma = Hp%mBod(1)*sqrt(Hp%Gm0mi(1)*X(1,1)) * (1.d0 - sqrt(1.d0 - X(2,1)**2))
     res = 2.d0*(X(6,2)+X(5,2)+X(4,2)) - (X(6,1)+X(5,1)+X(4,1)) - (X(5,1)+X(4,1)) 
     
     Z0(1) = sqrt(2.d0*Gamma/eta)*cos(res)
     Z0(2) = sqrt(2.d0*Gamma/eta)*sin(res)

     !-----------------------------------------------------------------------

     ! Parameters of the numerical integration
     Ip%t0      = t0
     Ip%tSpan   = tSpan*beta*eta
     Ip%Dt      = Dt*beta*eta
     Ip%tfac    = beta*eta
     Ip%Nsave   = Nsave

     !-----------------------------------------------------------------------

  end subroutine initInteg

!***************************************************************************************
end module parameters
