! Simple program for the propagation of two bodies in a 2:1 mean motion resonance
! course of Dinamica del Sistema Solare
program res12DSS

  use hamiltonian
  use integr

  implicit none

  character(256):: hamParamFile ! namelist containing the parameters of the Hamiltonian function
  character(256):: intParamFile ! namelist containing the parameters of the numerical integration
  character(256):: outFile      ! output file

  type(hamparam):: Hp  ! parameters of the Hamiltonian
  type(intparam):: Ip  ! parameters of the numerical integration

  double precision:: X(2) ! state vector
  double precision:: t ! time
  integer:: Nstep, Nreg

!---------------------------------------------------------------------------------------------
! Reading and computing the parameters

  if(COMMAND_ARGUMENT_COUNT() >= 3) then
     call GET_COMMAND_ARGUMENT(1,hamParamFile)
     call GET_COMMAND_ARGUMENT(2,intParamFile)
     call GET_COMMAND_ARGUMENT(3,outFile)
  else
     write(*,*) "Use: <program name> <parameterFile.nml> <initialConditions.nml> <outFile.dat>"
     stop
  endIf

  ! computing the parameters of the Hamiltonian
  call readHamParam(hamParamFile,trim(outFile)//"_ham.nml",Hp)

  ! computing the parameters of the numerical integration
  call initInteg(intParamFile,trim(outFile)//"_int.nml", Hp,Ip, X)

!---------------------------------------------------------------------------------------------
! Initializations

  t = Ip%t0
  Nstep = 0 ! number of timesteps realised

  ! Opening and writing the header of the output file
  open(newunit=Nreg,file=trim(outFile),status='REPLACE',action='WRITE') ! opening the output file
  write(Nreg,'(A)') "# x,y are the canonical coordinates of the 2:1 resonant model (M&D)"
  write(Nreg,'(A)') "# "

  write(Nreg,'(A)') "# t ; x ; y ; hamiltonian value; delta"

!---------------------------------------------------------------------------------------------
! Numerical integration

  call printFunc(X,t) ! saving the initial state
  call rkg(X, t, force, Ip%Dt, Ip%tSpan, printFunc) ! numerical integration

!---------------------------------------------------------------------------------------------

  close(Nreg) ! closing the output file

!---------------------------------------------------------------------------------------------
contains
!---------------------------------------------------------------------------------------------

  ! Force function for the numerical integration
  ! 
  function force(XX,tt)
     double precision,intent(in):: XX(:), tt
     double precision:: force(size(XX))

     double precision:: hamEq(2)

     ! Conservative part of the force + dissipation
     hamEq(:) = DHam(Hp,XX(1:2),tt)

     force(:) = hamEq(:)

  end function force

!---------------------------------------------------------------------------------------------

  ! Subroutine used after each step for saving the results.
  ! 
  subroutine printFunc(XX,tt)
     double precision,intent(in):: XX(:), tt

     double precision:: H

     if( mod(Nstep,Ip%Nsave) == 0 ) then

        H = Ham(Hp,XX,tt)

        ! coordinates of the 1-DOF integrable model (Murray-Dermott eq. 8.130)
        ! we rescale to the real time (not normalized)
        write(Nreg,'(ES22.15,*(1X,ES22.15))') tt/Ip%tfac, XX(:), H, Hp%delta+Hp%ddelta*tt

     endIf
     Nstep = Nstep + 1

  end subroutine printFunc

!---------------------------------------------------------------------------------------------

end program res12DSS
